package edu.cmu.cs.able.examples.RMI;
import java.rmi.*;
import java.util.*;
import javax.swing.*;

import java.net.*;
import java.io.*;

import edu.cmu.cs.able.gaugeInfrastructure.*;
import edu.cmu.cs.able.gaugeInfrastructure.util.*;
import edu.cmu.cs.able.gaugeInfrastructure.RMI.*;
import edu.cmu.cs.able.gaugeInfrastructure.RMI.Events.*;

public class GUIGaugeConsumer extends RMIGaugeConsumer {

    private GaugeConsumerGUI theGaugeConsumerGUI;   
        
    public GUIGaugeConsumer(String host)
    {
        super(host);
        try {
            gaugeBus = new RMIGaugeReportingBus (host);
            theGaugeConsumerGUI = new GaugeConsumerGUI(this);
            int p = Initialization.startGaugeConsumerService(this);         
            gaugeBus.registerGaugeConsumer(gaugeConsumerUID, p);            
            theGaugeConsumerGUI.display();
        }
        catch (Exception e) {
        }
    }


    public static void main(String args[]) {
        if (args.length < 1) return;
        Global.initLookAndFeel("Metal");
        GUIGaugeConsumer theGUIGaugeConsumer = new GUIGaugeConsumer(args[0]);
        //theAcmeGaugeConsumer.createGauge();
    }

    public void onReportValue(GaugeControl gauge, String valueName, String propertyName, String value)
    {
        theGaugeConsumerGUI.addReportedValue(gauge.getGaugeID().uid, valueName+value);
    }

    public void onReportConfigured(GaugeControl gauge, StringPairVector configParams)
    {
        theGaugeConsumerGUI.changeConfig(gauge.getGaugeID().uid, configParams);
    }

    /** Called when a created event is reported
     * @param gauge The gauge reporting the event
     */    
    public void onReportCreated(GaugeControl gauge)
    {
        RMIGauge g = (RMIGauge)gauge;
        theGaugeConsumerGUI.addGauge(g.getGaugeID(), g.configParams, g.setupParams);
    }

    public void onReportDeleted(GaugeControl gauge)
    {
        theGaugeConsumerGUI.deleteGauge(gauge.getGaugeID().uid);
    }
    public RMIGaugeManagerHandle getMgr(String gaugeType)
    {
        gaugeMgr = new RMIGaugeManagerHandle(busHost, gaugeType);
        return gaugeMgr;
    }
    
    /** 
    * Configures a gauge.
    * @param theGaugeID The GaugeID of the gauge being configured
    * @param configParams The configuration parameters
    */
    public void configureGauge(GaugeID theGaugeID, StringPairVector configParams)
    {   
        try {
            gaugeBus.configureGauge(theGaugeID, configParams);
        System.out.println("gaugeBus.configureGauge"+theGaugeID.toString()+configParams.toString());
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }

    /** 
    * Creates a gauge.
    * @param theGaugeID The GaugeID of the gauge being created
    * @param setupParams The setup parameters
    */
    public GaugeControl createGauge(GaugeID theGaugeID, StringPairVector setupParams, StringPairVector mappings)
    {
        try {
            gaugeMgr = getMgr(theGaugeID.gaugeType);
            return gaugeMgr.createGauge(theGaugeID, setupParams, mappings);
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    /** 
    * Deletes a gauge.
    * @param theGaugeID The GaugeID of the gauge being deleted
    */
    public void deleteGauge(GaugeControl gauge)
    {   
        try {
            gaugeMgr = getMgr(gauge.getGaugeID().gaugeType);
            gaugeMgr.deleteGauge(gauge);
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }
    
   
    /** 
    * Subscribe interest of the gauge consumer.
    * @param theGaugeID The GaugeID of the gauge to be observed
    * @param gaugeEvent The gauge events to be observed
    */
    public void subscribeInterest(GaugeID theGaugeID, GaugeEventType gaugeEvent)
    {
        System.out.println("to subscribe interest"+theGaugeID.toString());
        gaugeBus.subscribeInterest(gaugeConsumerUID, theGaugeID, gaugeEvent);
        System.out.println("subscribed interest"+theGaugeID.toString());
    }

    public void queryMetaInfo(String gaugeType, StringPairVector configParamsMeta, StringPairVector valuesMeta)
    {
        try {
            gaugeMgr = getMgr(gaugeType);
            System.out.println("querying meta:"+gaugeType);
            gaugeMgr.queryMetaInfo(gaugeType, configParamsMeta, valuesMeta);
            System.out.println(configParamsMeta.toString());
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }        
}
