/*
 * This class implements a RMI bandwidth gauge.
 * $Header: D:/Projects/Repository/GaugeAPI/edu/cmu/cs/able/gauges/RMI/RMIBandwidthGauge.java,v 1.6 2002/05/24 18:10:35 schmerl Exp $
 * $Log: RMIBandwidthGauge.java,v $
 * Revision 1.6  2002/05/24 18:10:35  schmerl
 * Removed delegation to GaugeImpl - this is now done in RMIGauge
 *
 * Revision 1.5  2002/02/20 20:41:29  schmerl
 * 1. Fixed gauge deletion in the Siena version so that deletes don't timeout.
 * 2. Changed gauge implementation pattern so that gauge computation does not need to be included in both RMII and Siena versions.
 *
 * Revision 1.4  2002/01/31 16:39:10  schmerl
 * Added CVS header information
 *
 *
 */

package edu.cmu.cs.able.gauges.RMI;
import java.rmi.*;
import java.util.*;

import java.net.*;
import java.io.*;

import edu.cmu.cs.able.gaugeInfrastructure.*;
import edu.cmu.cs.able.gauges.RemosBandwidthGaugeImpl;
import edu.cmu.cs.able.gaugeInfrastructure.RMI.*;
import edu.cmu.cs.able.gaugeInfrastructure.util.*;
import edu.cmu.cs.able.probes.RemosProbe.*;

public class RMIBandwidthGauge extends RMIGauge {
    
    protected String gaugeType = "RemosBandwidthGauge";
    private static String sienaHost = null;
    
    /** Creates new RMIBandwidthGauge
     * @param gaugeID The ID of the gauge to create.
     * @param creatorID The ID of the gauge manager that is creating the gauge.
     * @param setupParams The value of the setup params.
     */
    public RMIBandwidthGauge (GaugeID gaugeID, String creatorID, StringPairVector setupParams, StringPairVector mappings) {
        super (gaugeID, creatorID, setupParams, mappings);
        impl = new RemosBandwidthGaugeImpl (gaugeID, setupParams, mappings, gaugeBus, sienaHost);
    }
    
    /** A command-line interface to start the gauge.
     * @param args -gt <i>gauge type</i><br>
     * -gn <i>gauge name</i><br>
     * -mt <i>model type</i><br>
     * -mn <i>model name</i><br>
     * [-setup <i>name=value</i>]<br>
     * [-mappings <i>name=property</i>]<br>
     * -creator <i>creator ID</i><br>
     */
    public static void main (String args[]) {
        
        String gaugeName = "RMIBandwidthGaugeName";
        String gaugeType = "edu.cmu.cs.able.gauges.RMI.RMIBandwidthGauge";
        String modelName = "RMIBandwidthGaugeModelName";
        String modelType = "RMIBandwidthGaugeModelType";
        String creatorID = "";
        String gaugeUID = "";
        
        StringPairVector mappings = new StringPairVector ();
        StringPairVector setupParams = new StringPairVector ();
        boolean gotMappings = false;
        boolean gotSetup = false;
        // Process args
        boolean ok = true;
        for (int i = 0; i < args.length && ok; i++) {
            String tag = args [i];
            if (args [i].equals ("-creator")) {
                creatorID = args [++i];
            }
            else if (args [i].equals ("-gn")) {
                gaugeName = args [++i];
            }
            else if (args [i].equals ("-gt")) {
                gaugeType = args [++i];
            }
            else if (args [i].equals ("-uid")) {
                gaugeUID = args [++i];
            }
            else if (args [i].equals ("-mn")) {
                modelName = args [++i];
            }
            else if (args [i].equals ("-mt")) {
                modelType = args [++i];
            }
            else if (args [i].equals ("-setup")) {
                gotSetup = true;
                i++;
                while (i < args.length && args [i].charAt (0) != '-') {
                    String setup = args [i];
                    String name = setup.substring (0, setup.indexOf ('='));
                    String value = setup.substring (setup.indexOf ('=') + 1);
                    setupParams.addElement (name, value);
                    i++;
                }
                i--;
            }
            else if (args [i].equals ("-mappings")) {
                gotMappings = true;
                i++;
                while (i < args.length && args [i].charAt (0) != '-') {
                    String mapping = args [i];
                    String value = mapping.substring (0, mapping.indexOf ('='));
                    String property = mapping.substring (mapping.indexOf ('=') + 1);
                    mappings.addElement (value, property);
                    i++;
                }
                i--;
            }
            else if (args [i].equals ("-rmib")) {
                busHost = args [++i];
            }
            else if (args [i].equals ("-senp")) {
                sienaHost = args [++i];
            }
            else if (args [i].equals ("-debug")) {
                edu.cmu.cs.able.gaugeInfrastructure.util.Global.debugFlag = true;
            }
            else {
                ok = false;
            }
        }
        
        if (!ok) {
            printUsageMessage ();
            System.exit (0);
        }
        
        GaugeID gid = new GaugeID ();
        gid.modelName = modelName;
        gid.modelType = modelType;
        gid.gaugeName = gaugeName;
        gid.gaugeType = gaugeType;
        gid.uid = gaugeUID;
        System.out.println ("RMIBandwidthGauge started");
        System.out.flush ();
        
        RMIBandwidthGauge theBWGauge = new RMIBandwidthGauge (gid, creatorID, setupParams, mappings);
        theBWGauge.register ();
       
    }
    
    private static void printUsageMessage () {
        System.out.println ("Usage: RMIBandwidthGauge -creator <creatorID> -mt <modelType> -mn <modelName> -gt <gaugeType> -gn <gaugeName> -setup <name=value> -mappings <value=property> -rmib <rmibus> -senp <siena port>");
    }
    
}
