/*
 * $Header: D:/Projects/Repository/GaugeAPI/edu/cmu/cs/able/gauges/RMI/RMITimeGauge.java,v 1.6 2002/05/24 18:11:59 schmerl Exp $
 * $Log: RMITimeGauge.java,v $
 * Revision 1.6  2002/05/24 18:11:59  schmerl
 * Removed delegation from Gauges (now in RMI Gauge).
 * Also, removed classpath from gauge type.
 * Gauge managers now respond to gauges of a particular type,
 * and have hardwired internally the particular class to start.
 *
 * Revision 1.5  2002/02/20 20:41:29  schmerl
 * 1. Fixed gauge deletion in the Siena version so that deletes don't timeout.
 * 2. Changed gauge implementation pattern so that gauge computation does not need to be included in both RMII and Siena versions.
 *
 * Revision 1.4  2002/01/31 16:39:10  schmerl
 * Added CVS header information
 *
*/
package edu.cmu.cs.able.gauges.RMI;
import java.rmi.*;
import java.util.*;

import java.net.*;
import java.io.*;

import edu.cmu.cs.able.gaugeInfrastructure.*;
import edu.cmu.cs.able.gaugeInfrastructure.RMI.*;
import edu.cmu.cs.able.gaugeInfrastructure.util.*;
import edu.cmu.cs.able.gauges.*;

/** Implements a gauge that periodically reports the time.
 *
 * @author Hong Yan
 * @author Bradley Schmerl
 * @version 1.2
 * @ChangeLog
 * 01/29/2002 BRS  Added thread control and mapping
 *                to adhere to RMIGauge and the
 *                gauge infrastructure.
 * 02/20/2002 BRS Changed to correspond to new implementation pattern
 */
public class RMITimeGauge extends RMIGauge { 
    
    protected String gaugeType = "TimeGauge";
    
    /** Creates new RMITimeGauge
     * @param mappings Contains the mapping between each value and a higher level model
     * @param gaugeID The ID of the gauge to create.
     * @param creatorID The ID of the gauge manager that is creating the gauge.
     * @param setupParams The value of the setup params.
     */
    public RMITimeGauge (GaugeID gaugeID, String creatorID, StringPairVector setupParams, StringPairVector mappings) {
        super (gaugeID, creatorID, setupParams, mappings);
        impl = new TimeGaugeImpl (gaugeID, setupParams, mappings, gaugeBus);
        Global.debug (gaugeID.toString () + " created");
        Global.debug ("with");        
        //this.configParams.addElement ("frequency", new Integer (frequency).toString ());
        //this.setupParams.addElement ("frequency", new Integer (frequency).toString ());
        //this.configParams.addElement ("report", new Boolean (doReporting).toString ());
        int i;
        if (setupParams != null) {
            Global.debug(setupParams.toString());
       }
    }
    
    /** A command-line interface to start the gauge.
     * @param args -gt <i>gauge type</i><br>
     * -gn <i>gauge name</i><br>
     * -mt <i>model type</i><br>
     * -mn <i>model name</i><br>
     * [-setup <i>name=value</i>]<br>
     * [-mappings <i>name=property</i>]<br>
     * -creator <i>creator ID</i><br>
     */
    public static void main (String args[]) {
        
        String gaugeName = "RMITimeGaugeName";
        String gaugeType = "edu.cmu.cs.able.gauges.RMI.RMITimeGauge";
        String modelName = "RMITimeGaugeModelName";
        String modelType = "RMITimeGaugeModelType";
        String gaugeUID = "";
        String creatorID = "";
        
        StringPairVector mappings = new StringPairVector ();
        StringPairVector setupParams = new StringPairVector ();
        boolean gotMappings = false;
        boolean gotSetup = false;
        // Process args
        boolean ok = true;
        for (int i = 0; i < args.length && ok; i++) {
            String tag = args [i];
            if (args [i].equals ("-creator")) {
                creatorID = args [++i];
            }
            else if (args [i].equals ("-gn")) {
                gaugeName = args [++i];
            }
            else if (args [i].equals ("-gt")) {
                gaugeType = args [++i];
            }
            else if (args [i].equals ("-uid")) {
                gaugeUID = args [++i];
            }
            else if (args [i].equals ("-mn")) {
                modelName = args [++i];
            }
            else if (args [i].equals ("-mt")) {
                modelType = args [++i];
            }
            else if (args [i].equals ("-rmib")) {
                busHost = args [++i];
            }

            else if (args [i].equals ("-setup")) {
                gotSetup = true;
                i++;
                while (i < args.length && args [i].charAt (0) != '-') {
                    String setup = args [i];
                    String name = setup.substring (0, setup.indexOf ('='));
                    String value = setup.substring (setup.indexOf ('=') + 1);
                    setupParams.addElement (name, value);
                    i++;
                }
                i--;
            }
            else if (args [i].equals ("-mappings")) {
                gotMappings = true;
                i++;
                while (i < args.length && args [i].charAt (0) != '-') {
                    String mapping = args [i];
                    String value = mapping.substring (0, mapping.indexOf ('='));
                    String property = mapping.substring (mapping.indexOf ('=') + 1);
                    mappings.addElement (value, property);
                    i++;
                }
                i--;
            }
            else {
                ok = false;
            }
        }
        
        if (!ok) {
            printUsageMessage ();
            System.exit (0);
        }
        
        GaugeID gid = new GaugeID ();
        gid.modelName = modelName;
        gid.modelType = modelType;
        gid.gaugeName = gaugeName;
        gid.gaugeType = gaugeType;
        gid.uid = gaugeUID;
        
        System.out.println ("RMITimeGauge started");
        System.out.flush ();
        
        RMITimeGauge theTimeGauge = new RMITimeGauge (gid, creatorID, setupParams, mappings);
        theTimeGauge.register();
       
    }
    
    private static void printUsageMessage () {
        System.out.println ("Usage: RMITimeGauge -creator <creatorID> -mt <modelType> -mn <modelName> -gt <gaugeType> -gn <gaugeName> -setup <name=value> -mappings <value=property> -rmib <rmibus>");
    }
    
}
