/*
 * RemosBandwidthGaugeImpl.java
 *
 * Created on February 15, 2002, 3:13 PM
 */

package edu.cmu.cs.able.gauges;

import java.net.*;
import java.util.*;

import edu.cmu.cs.able.gaugeInfrastructure.*;
import edu.cmu.cs.able.gaugeInfrastructure.util.*;
import edu.cmu.cs.able.probes.RemosProbe.*;
/**
 *
 * @author  schmerl
 */
public class RemosBandwidthGaugeImpl extends GaugeImpl implements BandwidthProbeConsumer {
    
    private BandwidthProbeConsumer remosProbeConsumer;
    private ProbeManagerHandle pmh;
    private ProbeHandle prbHndl;
    
    // setup parameters
    protected InetAddress ip1 = null;
    protected InetAddress ip2 = null;
    protected String probeBus = null;
    
    // config parameters
    protected int probeFrequency = 500;
    
    protected boolean probesDeployed = false;
    protected boolean creationOK = true;
    
    protected Double currentBW = null;
    
    /** Creates a new instance of RemosBandwidthGaugeImpl */
    public RemosBandwidthGaugeImpl (GaugeID gid, StringPairVector setup, StringPairVector mappings, GaugeReportingBus bus, String probeBus) {
        super (gid, setup, mappings, bus);
        this.probeBus = probeBus;
        reportingFrequency = Integer.MAX_VALUE; // no reporting
        
        try {
            for (int i=0; i < setup.size (); i++) {
                if (setup.nameAt (i).toLowerCase ().equals ("ip_addr1")) {
                    ip1 = InetAddress.getByName (setup.valueAt (i));
                }
                else if (setup.nameAt (i).toLowerCase ().equals ("ip_addr2")) {
                    ip2 = InetAddress.getByName (setup.valueAt (i));
                }
                else if (setup.nameAt (i).toLowerCase ().equals ("frequency")) {
                    probeFrequency = Integer.parseInt (setup.valueAt (i));
                }
                else if (setup.nameAt (i).toLowerCase ().equals ("senp")) {
                    probeBus = setup.valueAt (i);
                }
            }
            
            probesDeployed = deployProbes ();
        }
        catch (Exception e) {
            creationOK = false;
        }
    }
    
    
    public void reportNewValue () {}
    
    public boolean configure (StringPairVector configParams) {
        boolean validConfiguration = true;
        String value = configParams.getValue ("Frequency");
        if (value == null) {
            value = configParams.getValue ("frequency");
        }
        Integer frequency = new Integer (value);
        if (frequency.intValue () <=0) {
            validConfiguration = false;
        }
        else {
            probeFrequency = frequency.intValue ();
            validConfiguration = prbHndl.focus (ip1, ip2, 1000/probeFrequency);
        }
        return validConfiguration;
    }
    
    public boolean queryAllValues (GaugeValueVector values) {
        if (currentBW != null) {
            values.addElement ("bandwidth", (String )mappings.get ("bandwidth"), currentBW.toString ());
            return true;
        }
        else 
            return false;
    
    }
    
    public boolean queryState (StringPairVector setupParams, StringPairVector configParams, StringPairVector mappings) {
        this.setupParams.copyInto (setupParams);
        configParams.addElement ("frequency", new Integer (probeFrequency).toString ());
        
        Enumeration enum = this.mappings.keys ();
        while (enum.hasMoreElements ()) {
            String key = (String )enum.nextElement ();
            mappings.addElement (key, (String )this.mappings.get (key));
        }
        return true;
    }
    
    public String queryValue (String valueName) {
        if (valueName.equals ("bandwidth")) {
            if (currentBW != null) {
                return currentBW.toString ();
            }
        }
        return null;
    }
    
    public boolean consistentlyCreated () {
        return probesDeployed && creationOK && ip1 != null && ip2 != null && probeBus != null;
    }
    
    // Probe handling
    private boolean deployProbes () {
        boolean probesDeployed = true;
        pmh = new ProbeManagerHandle (this, "myHandle", probeBus);
        prbHndl = pmh.deploy ();
        if (prbHndl == null || !prbHndl.activate (ip1, ip2, 1000/probeFrequency)) {
            probesDeployed = false;
        }
        return probesDeployed;
    }
    
    static final long EightMega = 1024*1024/8;
    
    public void sensed (double bw) {
        currentBW = new Double (bw/EightMega);
        gaugeBus.reportValue (gaugeID, "bandwidth", (String )mappings.get ("bandwidth"), currentBW.toString ());
    }
}
