/*
 * BandwidthGaugeImpl.java
 *
 * Created on February 20, 2002, 9:42 AM
 */
package edu.cmu.cs.able.gauges.Siena.Demo;

import edu.cmu.cs.able.gaugeInfrastructure.util.*;
import edu.cmu.cs.able.gaugeInfrastructure.*;
import edu.cmu.cs.able.probebus.*;
import edu.cmu.cs.able.probebus.siena.*;
import edu.cmu.cs.able.probebus.events.*;
import edu.cmu.cs.able.probebus.siena.events.*;
/** Implementation of the BandwidthGauge, conforming to gauge type:<br>
 * ServerLoadGauge = [<br>
 * &nbsp;&nbsp;type = edu.cmu.cs.able.gauges.Siena.Demo.BandwidthGauge<br>
 * &nbsp;&nbsp;setup = [where : String]<br>
 * &nbsp;&nbsp;config = []<br>
 * &nbsp;&nbsp;values = [bandwidth : float]
 * @author Bradley Schmerl
 * @version 1.1
 * @changelog BRS 2/20/2002 Created after reorganization of gauge implementations
 * @changelog SWC 4/19/2002 Removed cumulative and averaging of bandwidth.
 */
public class BandwidthGaugeImpl extends edu.cmu.cs.able.gaugeInfrastructure.GaugeImpl implements IProbeConsumer {
     /** Where (the component or connector) the gauge is attached. (maps to the probe shell name)
     */    
    protected String where = null;
    
    private static final Double ZERO = new Double (0.0);
    public static final double ALPHA = 0.75;
    /** The value reported by the gauge
     */
    Double bandwidth = ZERO;
    Double cumulative = ZERO;
    
    private Double lastReportedValue = ZERO;
    //int number = 0;  -zensoul, 4/19/2002
    int number = 1;
    String bwMapping = "";

    private static IProbeBus probeBus = null;
    private final String CONFIG_NAME = "BWConfig";
    private final String TARGET_CLASSES = "runtime.client.Client,runtime.client.ClientThread";
    public KXProbeHelper bwProbe;
    private IProbeEventFactory probeEventFactory = null;
    Boolean probesActivated = new Boolean (false);
    /** Creates new BandwidthGauge     
     * @param gaugeID The ID of the new gauge.
     * @param creatorID The ID of the gauge manager that created the gauge.
     * @param setupParams The parameters that the gauge uses to set itself up. The
     * valid setup parameters for this gauge are:<br>
     * &nbsp;&nbsp;where : String
     * @param mappings The property that the <pre>latency</pre> value is associated with.
     */
    public BandwidthGaugeImpl (GaugeID gid, StringPairVector setup, StringPairVector mappings, GaugeReportingBus gaugeBus, IProbeBus probeBus) {
        super (gid, setup, mappings, gaugeBus);
        reportingFrequency = 3000;
        Global.debug ("Creating BWGaugeImpl");
        this.probeBus = probeBus;
        where = setup.getValue ("where");
        bwMapping = (String )this.mappings.get ("bandwidth");
        probeEventFactory = new SienaProbeEventFactory ((SienaProbeBus )probeBus);
        doReporting = false;
        // Skipped the KXProbeHelper step as a hack, w/ assumption that all
        // probes have already been activated on system start
        //   (zensoul, May 03, 2002, HPDC paper deadline)
        // Then uncommented because gauge deletion stopped working properly for some reason
        bwProbe = new KXProbeHelper (CONFIG_NAME, where, TARGET_CLASSES, this, probeBus);
        //probesDeployed();
    }
    
    private void probesDeployed () {
        Global.debug ("Probes have been deployed");
        synchronized (probesActivated) {
            probesActivated = new Boolean (true);
        }
//        finalizeCreation (myManagerID, true);
        IProbeEvent e = probeEventFactory.createBandwidthEvent ("", CONFIG_NAME, where ); 
        probeBus.registerForEvent (e, this);
    }

    /** This method is used to configure the gauge.
     * @param configParams The parameters used to configure the gauge. For the
     * BandwidthGauge, there are no valid configuration parameters for this gauge.
     * @return Whether the gauge was successfully configured. Will always return false.
      */   
    public boolean configure (StringPairVector configParams) {
        return false;
    }
    
    
    public boolean consistentlyCreated () {
        boolean ret = false;
        synchronized (probesActivated) {
            ret = probesActivated.booleanValue ();
        }
        return ret;
    }

        /** Returns the current date.
     * @param values The vector into which to place the date.
     * @return Whether the gauge could report it's current value.
     */    
    public boolean queryAllValues (GaugeValueVector values) {
        synchronized (cumulative) {
            Double bandwidth = new Double (cumulative.doubleValue ()/number);
            values.addElement ("bandwidth", (String )mappings.get ("bandwidth"), bandwidth.toString ());
        }
        return true;
    }
 
    /** Returns the state of the gauge.
     * @param setupParams The vector into which the setup parameters are copied. Will be
     * empty for this gauge.
     * @param configParams The configuration parameters for this gauge. Will be empty.
     * @param mappings The property that each gauge value is associated with.
     * Will have an element for <pre>bandwidth</pre>.
     * @return Whether the state of the gauge could be queried.
     */    
    public boolean queryState (StringPairVector setupParams, StringPairVector configParams, StringPairVector mappings) {
        this.setupParams.copyInto (setupParams);
        configParams.removeAllElements ();
        
        java.util.Enumeration enum = this.mappings.keys ();
        while (enum.hasMoreElements ()) {
            String key = (String )enum.nextElement ();
            mappings.addElement (key, (String )this.mappings.get (key));
        }
        
        return true;
    }

     /** The method called to query the value of the gauge. This
     * provides a direct query for the gauge, as opposed to the
     * gauge reporting to value on the bus.
     * @param valueName The name of the value to query.
     * @return The corresponding value of the gauge. (Note: <I>it is assumed
     * that the querier know the property associated with this
     * value</I>).
     */    
    public String queryValue (String valueName) {
        if (valueName.equals ("bandwidth")) {
            String bw = "";
            synchronized (cumulative) {
                bw = Double.toString (cumulative.doubleValue ()/number);
            }
            return bw;
        }
        else {
            return "";
        }
    }
    
    public void reportNewValue () {
        Double lBandwidth = null;
        synchronized (cumulative) {
            if (number > 0) {
                lBandwidth = new Double (cumulative.doubleValue ()/number);
                //number = 0;  -zensoul, 4/19/2002
                //cumulative = ZERO;
            }
        }
        if (lBandwidth != null && !lBandwidth.equals (lastReportedValue)) 
            lastReportedValue = lBandwidth;
            gaugeBus.reportValue (gaugeID, "bandwidth", bwMapping, lBandwidth.toString ());
    }
    
    /** The callback invoked when a probe reports a single event.
     * @param e.  The event that was reported.
     */
    private boolean virgin = true;
    
    public void onProbeEvent (IProbeEvent event) {
        if (event instanceof SienaActivateResponseProbeEvent && virgin) {
            Global.debug ("Got activate response ... finished creation");
            probesDeployed ();
            virgin = false;
        }
        else if (event instanceof SienaBandwidthProbeEvent) {
            SienaBandwidthProbeEvent bwEvent = (SienaBandwidthProbeEvent )event;
            Global.debug ("Got probe event " + bwEvent.toString ());
            double newBandwidth = new Double (bwEvent.getProbeEvent ().getBandwidth ()).doubleValue ();
            synchronized (cumulative) {
                //number++;  -zensoul, 4/19/2002
                //cumulative = new Double (cumulative.doubleValue () + newBandwidth);
                cumulative = new Double(newBandwidth);
            }
            doReporting = true;
        }
    }
    
}
