/*
 * LatencyGaugeImpl.java
 *
 * Created on February 20, 2002, 2:22 PM
 */

package edu.cmu.cs.able.gauges.Siena.Demo;
import java.util.Enumeration;
import edu.cmu.cs.able.gaugeInfrastructure.Siena.*;
import edu.cmu.cs.able.gaugeInfrastructure.*;
import edu.cmu.cs.able.gaugeInfrastructure.util.*;
import edu.cmu.cs.able.gaugeInfrastructure.Events.*;
import edu.cmu.cs.able.probebus.*;
import edu.cmu.cs.able.probebus.siena.*;
import edu.cmu.cs.able.probebus.events.*;
import edu.cmu.cs.able.probebus.siena.events.*;
/** Implementation of the LatencyGauge, conforming to gauge type:<br>
 * LatencyGauge = [<br>
 * &nbsp;&nbsp;type = edu.cmu.cs.able.gauges.Siena.Demo.LatencyGauge<br>
 * &nbsp;&nbsp;setup = [where : String]<br>
 * &nbsp;&nbsp;config = [reset : boolean]<br>
 * &nbsp;&nbsp;values = [latency : float]
 * @author Bradley Schmerl
 * @version 1.0
 */
public class LatencyGaugeImpl extends edu.cmu.cs.able.gaugeInfrastructure.GaugeImpl implements IProbeConsumer {
    
    protected String where = null;
    public static final Double ZERO = new Double (0.0);
    Double latency = ZERO;
    private Double lastReportedValue = ZERO;
    final double ALPHA = 0.5;  // change from 0.75 to 0.1 on Apr 17, 2002
        // From excel analysis, alpha of 0.09 approximates to window of 60 samples
    
    private static IProbeBus probeBus = null;
    private final String CONFIG_NAME = "LatencyConfig";
    private final String TARGET_CLASSES = "runtime.client.Client,runtime.client.ClientThread";
    public KXProbeHelper latencyProbe;
    private IProbeEventFactory probeEventFactory = null;    
    Boolean probesActivated = new Boolean (false);
    /** Creates new SienaLatencyGauge     
     * @param gaugeID The ID of the new gauge.
     * @param creatorID The ID of the gauge manager that created the gauge.
     * @param setupParams The parameters that the gauge uses to set itself up. The
     * valid setup parameters for this gauge are:<br>
     * &nbsp;&nbsp;where : int
     * @param mappings The property that the <pre>latency</pre> value is associated with.
     */
    public LatencyGaugeImpl (GaugeID gid, StringPairVector setup, StringPairVector mappings, GaugeReportingBus gaugeBus, IProbeBus probeBus) {
        super (gid, setup, mappings, gaugeBus);
        // moved this earlier in the construction to ensure its effect when GaugeImpl has started reporting
        reportingFrequency = 3000;
        this.probeBus = probeBus;
        where = setup.getValue ("where");
        probeEventFactory = new SienaProbeEventFactory ((SienaProbeBus )probeBus);
        doReporting = false;
        // Skipped the KXProbeHelper step as a hack, w/ assumption that all
        // probes have already been activated on system start
        //   (zensoul, May 03, 2002, HPDC paper deadline)
        // Then uncommented because gauge deletion stopped working properly for some reason
        latencyProbe = new KXProbeHelper (CONFIG_NAME, where, TARGET_CLASSES, this, probeBus);
//        probesDeployed();
    }
    
    private void probesDeployed () {
        synchronized (probesActivated) {
            probesActivated = new Boolean (true);
        }
        
        IProbeEvent e = probeEventFactory.createLatencyEvent ("", CONFIG_NAME, where ); 
        probeBus.registerForEvent (e, this);
    }
    /** This method is used to configure the gauge.
     * @param configParams The parameters used to configure the gauge. For the
     * BandwidthGauge, there are no valid configuration parameters for this gauge.
     * @return Whether the gauge was successfully configured. Will always return false.
      */       
    public boolean configure (StringPairVector configParams) {
        return false;
    }
    
    public boolean consistentlyCreated () {
        boolean ret = false;
        synchronized (probesActivated) {
            ret = probesActivated.booleanValue ();
        }
        return ret;
    }
       /** Returns the current latency.
     * @param values The vector into which to place the latency.
     * @return Whether the gauge could report it's current value.
     */     
    public boolean queryAllValues (GaugeValueVector values) {
         synchronized (latency) {
            
            values.addElement ("latency", (String )mappings.get ("latency"), latency.toString ());
        }
        return true;       
    }
    
    public boolean queryState (StringPairVector setupParams, StringPairVector configParams, StringPairVector mappings) {
        this.setupParams.copyInto (setupParams);
        configParams.removeAllElements ();
        
        java.util.Enumeration enum = this.mappings.keys ();
        while (enum.hasMoreElements ()) {
            String key = (String )enum.nextElement ();
            mappings.addElement (key, (String )this.mappings.get (key));
        }
        
        return true;
    }
    
    public String queryValue (String valueName) {
        if (valueName.equals ("latency")) {
            String l = "";
            synchronized (latency) {
                l = latency.toString ();
            }
            return l;
        }
        else {
            return "";
        }
    }
    
    public void reportNewValue () {
        Double lLatency = null;
        synchronized (latency) {
            lLatency = latency;
        }
        if (!lLatency.equals (lastReportedValue)) {
            lastReportedValue = lLatency;
            gaugeBus.reportValue (gaugeID, "latency",
                                (String )mappings.get ("latency"),
                                lLatency.toString ());
        }
    }
    
    private boolean virgin = true;
    
    /** The callback invoked when a probe reports a single event.
     * @param event.  The event that was reported.
     */
    public void onProbeEvent (IProbeEvent event) {
        if (event instanceof SienaActivateResponseProbeEvent && virgin) {
            Global.debug ("Got activate response ... finishing creation");
            probesDeployed ();
            virgin = false;
        }
        else if (event instanceof SienaLatencyProbeEvent) {
            SienaLatencyProbeEvent latencyEvent =
                    (SienaLatencyProbeEvent )event;
            doReporting = true;
            //Avg = alpha * curVal + (1 - alpha) * Avg
            double newLatency = new Double (latencyEvent.getProbeEvent ().getLatency ()).doubleValue ();
            synchronized (latency) {
                latency = new Double ((ALPHA * newLatency) + ((1 - ALPHA) * this.latency.doubleValue ()));
            }
  
        }
    }
    
}
