/*
 * SienaTimeGauge.java
 *
 * Created on November 6, 2001, 2:29 PM
 * $Header: D:/Projects/Repository/GaugeAPI/edu/cmu/cs/able/gauges/Siena/SienaTimeGauge.java,v 1.5 2002/05/24 18:13:32 schmerl Exp $
 * $Log: SienaTimeGauge.java,v $
 * Revision 1.5  2002/05/24 18:13:32  schmerl
 * Removed delegation to GaugeImpl in gauges - now done by SienaGauge
 * Removed full path to class form gauge type in gauge managers and gauges.
 * Gauge managers know where the class is to implement the gauge.
 *
 * Revision 1.4  2002/05/15 16:28:25  schmerl
 * Changed gauge type so that the Java path is no longer there.
 *
 * Revision 1.3  2002/02/20 20:41:32  schmerl
 * 1. Fixed gauge deletion in the Siena version so that deletes don't timeout.
 * 2. Changed gauge implementation pattern so that gauge computation does not need to be included in both RMII and Siena versions.
 *
 * Revision 1.2  2002/01/31 16:39:11  schmerl
 * Added CVS header information
 *
 */

package edu.cmu.cs.able.gauges.Siena;
import edu.cmu.cs.able.gauges.*;
import edu.cmu.cs.able.gaugeInfrastructure.util.*;
import edu.cmu.cs.able.gaugeInfrastructure.*;
import java.util.*;
/** This gauge provides an example that periodically reports
 * the date and time to the gauge bus. Its type is:<br>
 * TimeGauge = [<br>
 * &nbsp;&nbsp;type = edu.cmu.cs.able.gauges.Siena.SienaTimeGauge,<br>
 * &nbsp;&nbsp;setup = []<br>
 * &nbsp;&nbsp;config = [frequency : int; report : boolean [default = false;]]<br>
 * &nbsp;&nbsp;values = [date : String]<br>
 *
 * @author Bradley Schmerl
 * @version 1.1
 */
public class SienaTimeGauge extends edu.cmu.cs.able.gaugeInfrastructure.Siena.SienaGauge {

    
    protected String gaugeType = "TimeGauge";

    /** Creates new SienaTimeGauge
     * @param gaugeID The ID of the new gauge.
     * @param creatorID The ID of the gauge manager that created the gauge.
     * @param setupParams The parameters that the gauge uses to set itself up. The
     * valid setup parameters for this gauge are:<br>
     * &nbsp;&nbsp;frequency : int
     * @param mappings The property that the <pre>date</pre> value is associated with.
     */
    public SienaTimeGauge(GaugeID gaugeID, String creatorID, StringPairVector setupParams, StringPairVector mappings) {
        super (gaugeID, creatorID, setupParams, mappings);
        Global.debug ("Creating implementation");
        impl = new TimeGaugeImpl (gaugeID, setupParams, mappings, gaugeBus);
        
        Global.debug ("Finalizing creation");
        finalizeCreation (creatorID, impl.consistentlyCreated ());
    }

    /** A command-line interface to start the gauge.
     * @param args -gt <i>gauge type</i><br>
     * -gn <i>gauge name</i><br>
     * -mt <i>model type</i><br>
     * -mn <i>model name</i><br>
     * [-setup <i>name=value</i>]<br>
     * [-mappings <i>name=property</i>]<br>
     * -creator <i>creator ID</i><br>
     * -senp <i>siena port</i><br>
     * [-debug]
     */    
    public static void main (String [] args) {

        
        String creatorID = null;
        String gaugeName = null;
        String gaugeType = null;
        String modelName = null;
        String modelType = null;
        StringPairVector mappings = new StringPairVector ();
        StringPairVector setupParams = new StringPairVector ();
        boolean gotMappings = false;
        boolean gotSetup = false;
        // Process args
        boolean ok = true;
        for (int i = 0; i < args.length && ok; i++) {
            String tag = args [i];
            if (args [i].equals ("-creator")) {
                creatorID = args [++i];
            }
            else if (args [i].equals ("-gn")) {
                gaugeName = args [++i];
            }
            else if (args [i].equals ("-gt")) {
                gaugeType = args [++i];
            }
            else if (args [i].equals ("-mn")) {
                modelName = args [++i];
            }
            else if (args [i].equals ("-mt")) {
                modelType = args [++i];
            }
            else if (args [i].equals ("-setup")) {
                gotSetup = true;
                i++;
                while (i < args.length && args [i].charAt (0) != '-') {
                    String setup = args [i];
                    String name = setup.substring (0, setup.indexOf ('='));
                    String value = setup.substring (setup.indexOf ('=') + 1);
                    setupParams.addElement (name, value);
                    i++;
                } 
                i--;
            }
            else if (args [i].equals ("-mappings")) {
                gotMappings = true;
                i++;
                while (i < args.length && args [i].charAt (0) != '-') {
                    String mapping = args [i];
                    String value = mapping.substring (0, mapping.indexOf ('='));
                    String property = mapping.substring (mapping.indexOf ('=') + 1);
                    mappings.addElement (value, property);
                    i++;
                }
                i--;
            }
            else if (args [i].equals ("-senp")) {
                edu.cmu.cs.able.gaugeInfrastructure.Siena.Initialization.initSiena (args [++i], edu.cmu.cs.able.gaugeInfrastructure.Siena.Initialization.GAUGE);
            }
            else if (args [i].equals ("-debug")) {
                edu.cmu.cs.able.gaugeInfrastructure.util.Global.debugFlag = true;
            }
            else {
                ok = false;
            }            
        }
        
        if (!ok) {
            printUsageMessage ();
            System.exit (0); 
        }
        
        GaugeID gid = new GaugeID ();
        gid.modelName = modelName;
        gid.modelType = modelType;
        gid.gaugeName = gaugeName;
        gid.gaugeType = gaugeType;
   
       
        SienaTimeGauge gauge = new SienaTimeGauge (gid, creatorID, setupParams, mappings);
       
        
    }
    
    private static void printUsageMessage () {
        System.out.println ("Usage: SienaTimeGauge -creator <creatorID> -mt <modelType> -mn <modelName> -gt <gaugeType> -gn <gaugeName> -setup <name=value> -mappings <value=property> -senp <sienaport>");
        
    }
    
}
