/*
 * TimeGaugeImpl.java
 *
 * Created on February 15, 2002, 4:44 PM
 */

package edu.cmu.cs.able.gauges;
import edu.cmu.cs.able.gaugeInfrastructure.util.*;
import edu.cmu.cs.able.gaugeInfrastructure.*;
import java.util.*;
/**
 *
 * @author  schmerl
 */
public class TimeGaugeImpl extends GaugeImpl {
    
    Date date;
    int frequency;
    
    /** Creates a new instance of TimeGaugeImpl */
    public TimeGaugeImpl (GaugeID gid, StringPairVector setupParams, StringPairVector mappings, GaugeReportingBus bus) {
        super (gid, setupParams, mappings, bus);

        frequency = Integer.parseInt (setupParams.getValue ("frequency"));
        Global.debug ("Setting frequency to " + frequency);
        reportingFrequency = frequency;
        if (setupParams.getValue ("report").equals ("true")) {
            doReporting = true;
        }
        else {
            doReporting = false;
        }
    }
    
    public boolean consistentlyCreated () {
        return frequency > 0;
    }
    
        /** This method is used to configure the gauge.
     * @param configParams The parameters used to configure the gauge. For the
     * SienaTimeGauge, the valid configuration parameters are:<br>
     * &nbsp;&nbsp;frequency : int (millisecond period of reporting)<br>
     * &nbsp;&nbsp;report : true (whether the gauge should report a value)
     * @return Whether the gauge was successfully configured.
      */    
    public boolean configure(StringPairVector configParams) {
        for (int i = 0; i < configParams.size (); i++) {
            if (configParams.nameAt (i).equals ("frequency")) {
                frequency = Integer.parseInt (configParams.valueAt (i));
                reportingFrequency = frequency;
            }
            else if (configParams.nameAt (i).equals ("report")) {
                if (configParams.valueAt (i).equals ("true"))
                    doReporting = true;
                else doReporting = false;
            }
        }
        return true;
    }
    
    /** Returns the current date.
     * @param values The vector into which to place the date.
     * @return Whether the gauge could report it's current value.
     */    
    public boolean queryAllValues(GaugeValueVector values) {
        values.addElement ("date", (String )mappings.get ("date"), date.toString ());
        return true;
    }
    
    /** Returns the state of the gauge.
     * @param setupParams The vector into which the setup parameters are copied. Will be
     * empty for this gauge.
     * @param configParams The configuration parameters for this gauge. Will have
     * an element for <pre>frequency</pre> and <pre>report</pre>.
     * @param mappings The property that each gauge value is associated with.
     * Will have an element for <pre>date</pre>.
     * @return Whether the state of the gauge could be queried.
     */    
    public boolean queryState(StringPairVector setupParams, StringPairVector configParams, StringPairVector mappings) {
        this.setupParams.copyInto (setupParams);
        configParams.addElement ("frequency", new Integer (frequency).toString ());
        configParams.addElement ("report", new Boolean (doReporting).toString ());
        
        Enumeration enum = this.mappings.keys ();
        while (enum.hasMoreElements ()) {
            String key = (String )enum.nextElement ();
            mappings.addElement (key, (String )this.mappings.get (key));
        }
        
        return true;
    }
    
    /** The method called to query the value of the gauge. This
     * provides a direct query for the gauge, as opposed to the
     * gauge reporting to value on the bus.
     * @param valueName The name of the value to query.
     * @return The corresponding value of the gauge. (Note: <I>it is assumed
     * that the querier know the property associated with this
     * value</I>).
     */    
    public String queryValue(String valueName) {
        if (valueName.equals ("date")) {
            return date.toString ();
        }
        else {
            return "";
        }
    }
    
    /** This method is called periodically by the reporting thread
     * to generate a new value and report it. It is inherited from
     * Siena gauge.
     */    
    public void reportNewValue() {
        date = new Date ();
        gaugeBus.reportValue (gaugeID, "date", (String )mappings.get ("date"), date.toString ());
    }
    
}
